' <clown-build> vbc /d:DEBUG=True /t:WinExe /r:System.dll,System.Drawing.dll,System.Windows.Forms.dll,Gregor.Core.dll,Gregor.UICore.dll,Gregor.AppCore.dll AssemblyBrowser.vb </clown-build>

Option Strict On

Imports System
Imports System.Collections.Specialized
Imports System.Text
Imports System.Windows.Forms
Imports Gregor.Core
Imports Gregor.AppCore
Imports Gregor.AppCore.Extensions

Namespace AssemblyBrowser

Public Module M

    Private s_Form As FAssemblyBrowser
    Private s_TitleBuilder As StringBuilder
    Private s_CodeOptions As CCodeOptions

    Public Sub Main(ByVal args() As String)

        Dev.StopOnExceptions = True
        Dev.AllowDetails = True

        ' browser site provides assembly selection, images, etc.
        ' by default, the current directory and the framework directory are searched
        ' you can add additional search folders to site.PathSearcher.Paths
        Dim site As New CDefaultAssemblyBrowserSite()

        ' create form, handle selection events
        s_Form = New FAssemblyBrowser(site)
        s_Form.Icon = AppCoreStockIcons.ImageList.GetIcon("Library.ico")
        AddHandler s_Form.TypeSelectionChanged, AddressOf HandleBrowserSelectionChanged
        AddHandler s_Form.MemberSelectionChanged, AddressOf HandleBrowserSelectionChanged
        s_TitleBuilder = New StringBuilder(64)
        s_CodeOptions = New CCodeOptions()
        s_CodeOptions.UseAliasNames = False
        s_CodeOptions.UseFullNames = True

        ' add shortcuts to popular assemblies
        s_Form.RecentAssemblyFileNames.Push("System.XML")
        s_Form.RecentAssemblyFileNames.Push("System.Windows.Forms")
        s_Form.RecentAssemblyFileNames.Push("System.Drawing")
        s_Form.RecentAssemblyFileNames.Push("System.Data")
        s_Form.RecentAssemblyFileNames.Push("System")
        s_Form.RecentAssemblyFileNames.Push("mscorlib")

        ' use a given language (default: CVbNetSyntaxProvider)
        s_Form.SyntaxProvider = new CCSharpSyntaxProvider()

        ' a DetailsCallback delegate provides additional informational
        ' text when an item is selected, such as XML documentation
        s_Form.DetailsCallback = Nothing

        ' process command line
        If args.Length > 0 AndAlso "/?".Equals(args(0)) Then
            Dim sb As New StringBuilder()
            sb.Append("Syntax:")
            sb.Append(Environment.NewLine)
            sb.Append("    AssemblyBrowser [command]+")
            sb.Append(Environment.NewLine)
            sb.Append("Where:")
            sb.Append(Environment.NewLine)
            sb.Append("    command ::= load:<assembly-name>")
            sb.Append(" | type:<full-type-name>")
            sb.Append(" | member:<member-name-or-pattern>")
            MessageBox.Show(sb.ToString(), "AssemblyBrowser", MessageBoxButtons.OK, MessageBoxIcon.Information)
            Exit Sub
        Else
            Dim sArg As String
            For Each sArg In args
                Try
                    s_Form.RunCommand(sArg)
                Catch ex As Exception
                    Dev.ProcessException(ex)
                End Try
            Next sArg
        End If

        ' run
        Application.Run(s_Form)

    End Sub

    Private Sub HandleBrowserSelectionChanged(ByVal sender As Object, ByVal e As EventArgs)
        Try
            Dim assyInfo As CAssemblyInfoEx = s_Form.SelectedAssembly
            Dim typeInfo As CTypeInfoEx = s_Form.SelectedType
            s_TitleBuilder.Remove(0, s_TitleBuilder.Length)
            If Not (assyInfo Is Nothing) Then
                s_TitleBuilder.Append("[")
                s_TitleBuilder.Append(assyInfo.Name)
                s_TitleBuilder.Append("]")
            End If
            If Not (typeInfo Is Nothing) Then
                If s_TitleBuilder.Length > 0 Then
                    s_TitleBuilder.Append(" ")
                End If
                s_TitleBuilder.Append(Reflect.GetTypeName(typeInfo.InnerType, s_Form.SyntaxProvider, s_CodeOptions))
                s_TitleBuilder.Append(" ")
            End If
            If s_TitleBuilder.Length > 0 Then
                s_TitleBuilder.Append(" - ")
            End If
            s_TitleBuilder.Append("Assembly Browser")
            s_Form.Text = s_TitleBuilder.ToString()
        Catch ex As Exception
            Dev.ProcessException(ex, True)
        End Try
    End Sub

End Module

End Namespace
