Option Strict On

Imports System
Imports System.Collections
Imports System.IO

Namespace BabaJaga

Public Module MStart

Private m_Preview As Boolean
Private m_MinDate As Date
Private m_DirCreationCount As Int32
Private m_FileCopyCount As Int32
Private m_PatternCountTotals As Hashtable
Private m_FileCopySize As Int32
Private m_PatternSizeTotals As Hashtable

Public Sub Main(ByVal args() As String)
    ' hello
    Log("BabaJaga - Backup Utility")
    System.Threading.Thread.CurrentThread.Sleep(1000)
    ' check
    If args.Length < 5 OrElse args(0) <> "/preview" AndAlso args(0) <> "/real" Then
        Log("Syntax:")
        Log("    BabaJaga /preview | /real <source-dir> <target-dir> <file-patterns> <min-date>")
        Exit Sub
    End If
    ' create helpers
    m_PatternCountTotals = New Hashtable(36)
    m_PatternSizeTotals = New Hashtable(36)
    ' feedback
    m_Preview = (args(0) = "/preview")
    Try
        m_MinDate = Date.Parse(args(4))
    Catch ex As Exception
        Log("Can't parse min-date: " & ex.Message)
        Exit Sub
    End Try
    Dim sourceDir As New DirectoryInfo(args(1))
    Dim targetDir As New DirectoryInfo(args(2))
    Dim sFilePatterns As String = args(3)
    Log("    Preview : " & m_Preview)
    Log("    Source  : " & sourceDir.FullName)
    Log("    Target  : " & targetDir.FullName)
    Log("    Patterns: " & sFilePatterns)
    Log("    MinDate : " & m_MinDate)
    System.Threading.Thread.CurrentThread.Sleep(1000)
    ' work
    Try
        Dim asFilePatterns() As String = sFilePatterns.Split(";"c)
        ProcessDirectory(sourceDir, targetDir, asFilePatterns)
    Catch ex As Exception
        Log(ex)
    End Try
    ' done
    System.Threading.Thread.CurrentThread.Sleep(1000)
    Log("Totals:")
    Log("    " & m_FileCopyCount & " files copied (" & m_FileCopySize & " bytes)")
    Dim di As DictionaryEntry
    For Each di In m_PatternCountTotals
        Dim size As Int32 = DirectCast(m_PatternSizeTotals(di.Key), Int32)
        Log("        " & di.Value.ToString() & " files matching " & di.Key.ToString() & " (" & size & " bytes)")
    Next di
    Log("    " & m_DirCreationCount & " directories created (directly)")
    Log("Done")
End Sub

Private Sub ProcessDirectory(ByVal sourceDir As DirectoryInfo, ByVal targetDir As DirectoryInfo, ByVal asFilePatterns() As String)
    Log("Processing " & sourceDir.FullName & " ...")
    ' get files
    Dim sFilePattern As String
    For Each sFilePattern In asFilePatterns
        Dim sourceFiles() As FileInfo = sourceDir.GetFiles(sFilePattern)
        sourceFiles = FilterFilesByMinDate(sourceFiles)
        ' any matches?
        If sourceFiles.Length > 0 Then
            ' ensure target directory exists if files match
            If Not targetDir.Exists() Then
                Log("    Creating " & targetDir.FullName)
                If Not m_Preview Then
                    targetDir.Create()
                    targetDir.Refresh() ' flushes the dir info to disk, so next time it's recognized
                End If
                m_DirCreationCount += 1
            End If
            ' copy files
            Log("    Copying " & sourceFiles.Length & " files matching " & sFilePattern)
            Dim sourceFile As FileInfo, size As Int32 = 0, count As Int32 = 0
            For Each sourceFile In sourceFiles
                Dim sTargetFile As String = Path.Combine(targetDir.FullName, sourceFile.Name)
                If Not m_Preview Then
                    sourceFile.CopyTo(sTargetFile, True)
                End If
                count += 1
                size += Convert.ToInt32(sourceFile.Length)
            Next sourceFile
            ' statistics
            m_FileCopyCount += count
            AddToInt32Hashtable(m_PatternCountTotals, sFilePattern, sourceFiles.Length)
            m_FileCopySize += size
            AddToInt32Hashtable(m_PatternSizeTotals, sFilePattern, size)
        End If
    Next sFilePattern
    ' recurse
    Dim subSourceDir As DirectoryInfo, subSourceDirs() As DirectoryInfo
    subSourceDirs = sourceDir.GetDirectories()
    Array.Sort(subSourceDirs, New CDirectoryComparer())
    For Each subSourceDir In subSourceDirs
        Dim sSubTargetDir As String = Path.Combine(targetDir.FullName, subSourceDir.Name)
        Dim subTargetDir As New DirectoryInfo(sSubTargetDir)
        ProcessDirectory(subSourceDir, subTargetDir, asFilePatterns)
    Next subSourceDir
End Sub

Private Function FilterFilesByMinDate(ByVal files() As FileInfo) As FileInfo()
    If Not m_MinDate.Equals(Date.MinValue) Then
        Dim list As New ArrayList(files.Length)
        Dim fi As FileInfo
        For Each fi In files
            If fi.LastWriteTime.CompareTo(m_MinDate) >= 0 Then
                list.Add(fi)
            End If
        Next fi
        files = New FileInfo(list.Count - 1){}
        list.CopyTo(files, 0)
    End If
    Return files
End Function

Private Sub AddToInt32Hashtable(ByVal table As Hashtable, ByVal sKey As String, ByVal i As Int32)
    Dim total As Int32 = 0
    If table.ContainsKey(sKey) Then
        total = DirectCast(table(sKey), Int32)
    End If
    table(sKey) = total + i
End Sub

Private Sub Log(ByVal o As Object)
    Console.WriteLine(o)
End Sub

End Module

Friend Class CDirectoryComparer
        Implements IComparer

Public Sub New()
    MyBase.New()
End Sub

Public Function Compare(ByVal a As Object, ByVal b As Object) As Int32 Implements IComparer.Compare
    Dim dirA As DirectoryInfo = DirectCast(a, DirectoryInfo)
    Dim dirB As DirectoryInfo = DirectCast(b, DirectoryInfo)
    Return String.Compare(dirA.Name, dirB.Name)
End Function

End Class

End Namespace
