Option Strict On

Imports System
Imports System.Collections
Imports System.IO
Imports System.Text

Namespace Build

Public Module MStart

#Region "Constants"
Private Const TARGETTYPE As String = "TargetType"
Private Const OUTFILENAME As String = "OutFileName"
Private Const REFERENCES As String = "References"
Private Const IMPORTLIST As String = "Imports"
Private Const SOURCEFILES As String = "SourceFiles"
#End Region

' the application's entry point
Public Sub Main(ByVal args() As String)
    If args.Length < 1 Then
        Console.WriteLine("Syntax:")
        Console.WriteLine("    Build <new-or-existing-config-file>")
        Exit Sub
    End If
    Try
        If Not File.Exists(args(0)) Then
            CreateConfigFile(args(0))
        End If
        Dim params As New Hashtable()
        ReadConfigFile(args(0), params)
        CallVbc(args(0), _
                DirectCast(params(TARGETTYPE), String), _
                DirectCast(params(OUTFILENAME), String), _
                DirectCast(params(REFERENCES), String), _
                DirectCast(params(IMPORTLIST), String), _
                GetSourceFiles(Environment.CurrentDirectory, DirectCast(params(SOURCEFILES), String)))
    Catch ex As Exception
        Console.WriteLine(ex.Message)
    End Try
End Sub

Private Sub CreateConfigFile(ByVal sFileName As String)
    Dim writer As StreamWriter, stream As FileStream
    Try
        stream = File.OpenWrite(sFileName)
        writer = New StreamWriter(stream)
        writer.WriteLine(CreateConfigFileLine(TARGETTYPE))
        writer.WriteLine(CreateConfigFileLine(OUTFILENAME))
        writer.WriteLine(CreateConfigFileLine(REFERENCES))
        writer.WriteLine(CreateConfigFileLine(IMPORTLIST))
        writer.WriteLine(CreateConfigFileLine(SOURCEFILES))
        writer.Flush()
    Finally
        If Not stream Is Nothing Then
            stream.Close()
        End If
    End Try
End Sub

Private Function CreateConfigFileLine(ByVal sParam As String) As String
    Dim sLine As String = sParam & "="
    Console.Write(sLine)
    sLine &= Console.ReadLine()
    Return sLine
End Function

Private Sub ReadConfigFile(ByVal sFileName As String, ByVal params As Hashtable)
    Dim reader As StreamReader, stream As FileStream
    Try
        stream = File.OpenRead(sFileName)
        reader = New StreamReader(stream)
        Dim sLine As String
        Do
            sLine = reader.ReadLine
            If sLine Is Nothing Then
                Exit Do
            Else
                Dim asNV() As String = sLine.Split("="C)
                If asNV.Length = 2 AndAlso asNV(0) <> String.Empty Then
                    params.Add(asNV(0), asNV(1))
                End If
            End If
        Loop Until sLine Is Nothing
    Finally
        If Not stream Is Nothing Then
            stream.Close()
        End If
    End Try
End Sub

Private Function GetSourceFiles(ByVal sDir As String, ByVal sFilterList As String) As String
    Dim sb As New StringBuilder()
    Dim fil As FileInfo, dir As New DirectoryInfo(sDir)
    If sFilterList Is Nothing Then
        Throw New Exception("No files set.")
    End If
    Dim asFilters() As String = sFilterList.Split(";"C)
    For Each fil In dir.GetFiles()
        Dim sFilter As String
        For Each sFilter In asFilters
            If fil.Name Like sFilter Then
                sb.Append(" ")
                sb.Append(fil.Name)
                Exit For
            End If
        Next sFilter
    Next fil
    Return sb.ToString()
End Function

Private Sub CallVbc(ByVal sConfigFileName As String, ByVal sTargetType As String, ByVal sOutFileName As String, ByVal sReferences As String, ByVal sImports As String, ByVal sSourceFiles As String)
    Dim s As String = "/optionstrict /warnaserror "
    If sTargetType <> String.Empty Then
        s &= "/target:" & sTargetType & " "
    End If
    If sOutFileName <> String.Empty Then
        s &= "/out:" & sOutFileName & " "
    Else
        Dim i As Integer = sConfigFileName.LastIndexOf(".")
        If i > 0 Then
            sConfigFileName = sConfigFileName.Substring(0, i)
        End If
        s &= "/out:" & sConfigFileName & " "
    End If
    If sReferences <> String.Empty Then
        s &= "/reference:" & sReferences & " "
    End If
    If sImports <> String.Empty Then
        s &= "/imports:" & sImports & " "
    End If
    s &= sSourceFiles
    Microsoft.VisualBasic.Shell("vbc " & s, , True)
End Sub

End Module

End Namespace