' build:
' vbc.exe /debug+ /r:System.dll,Gregor.Core.dll Clown.vb
' <shell:vbc.exe /debug+ /r:System.dll,Gregor.Core.dll Clown.vb>

Option Strict On

Imports System
Imports System.Net
Imports System.Collections
Imports System.IO
Imports System.Text
Imports Microsoft.VisualBasic
Imports Gregor.Core

Namespace Clown

Public Module M

    Private Const SYNOPSIS As String = "Synopsis:" & vbCrLf & "    Clown - builds from arbitrary sources, and runs programs."
    Private s_SynopsisLogged As Boolean

    Public Sub Main(ByVal args() As String)
        Try
            Run(args)
        Catch ex As Exception
            W(ex)
        End Try
    End Sub

    Private Sub Run(ByVal args() As String)
        ' setup supported commands
        Dim info As New CClownInfo()
        info.Commands.Add(New CCommandInfo("build", "clown-build", "// <clown-build> csc.exe /debug+ /r:MyLib.dll Source.cs </clown-build>"))
        info.Commands.Add(New CCommandInfo("run", "clown-run", "// <clown-run> Source.exe /nologo /realmode </clown-run>", True))
        ' parse arguments
        Dim params As CClownParams = ParseArgs(info, args)
        ' go
        Log(SYNOPSIS, params.Silent)
        s_SynopsisLogged = True
        ' read configured commands
        If params.IsUrl Then
            ' download remote documents
            Dim sContents As String = ReadRemoteClownInfo(info, params)
            SaveToTempFile(info, params, sContents) ' changes FileOrUrl
        Else
            ReadLocalClownInfo(info, params)
        End If
        ' try guessing defaults?
        If info.IsEmpty() Then
            If Not GuessDefaultCommands(info, params) Then
                SyntaxExit(info, "No default commands for this file type, and no commands given in the file.")
            End If
        End If
        ' fixup code?
        If Not (params.FixupInfo Is Nothing) Then
            FixupCode(info, params)
        End If
        ' run configured/guessed commands
        RunCommands(info, params)
    End Sub

    Private Function ParseArgs(ByVal info As CClownInfo, ByVal args() As String) As CClownParams
        Dim ret As New CClownParams(args)
        If args.Length < 1 Then
            SyntaxExit(info, "No arguments provided.")
        End If
        Dim iArg As Integer
        For iArg = 0 To args.Length - 1
            Dim sArg As String = args(iArg)
            If ret.FileOrUrl Is Nothing Then ' no more checks after file arg
                If sArg.StartsWith("/") Then
                    Select Case sArg
                        Case "/norun"
                            ret.SkipList &= " clown-run "
                        Case "/nobuild"
                            ret.SkipList &= " clown-build "
                        Case "/silent"
                            ret.Silent = True
                        Case "/fixup:member"
                            If ret.FixupInfo Is Nothing Then
                                ret.FixupInfo = New CFixupInfo()
                            End If
                            ret.FixupInfo.MemberOptions = MemberFixupOptions.Main
                        Case "/fixup:type"
                            If ret.FixupInfo Is Nothing Then
                                ret.FixupInfo = New CFixupInfo()
                            End If
                            ret.FixupInfo.TypeOptions = TypeFixupOptions.Module
                            ret.FixupInfo.FileOptions = FileFixupOptions.Imports
                        Case Else
                            SyntaxExit(info, "Option '" & sArg & "' not supported.")
                    End Select
                Else
                    If sArg.StartsWith("http") Then
                        ret.IsUrl = True
                    ElseIf Not File.Exists(sArg) Then
                        SyntaxExit(info, "File '" & sArg & "' not found.")
                    End If
                    ret.FileOrUrl = sArg
                    ret.FileArgIndex = iArg
                End If
            End If
        Next iArg
        If ret.FileOrUrl Is Nothing Then
            SyntaxExit(info, "No file/url argument provided.")
        End If
        Return ret
    End Function

    Private Sub SyntaxExit(ByVal info As CClownInfo, ByVal sMessage As String)
        If Not s_SynopsisLogged Then
            W(SYNOPSIS)
        End If
        W("Syntax:")
        W("    Clown [<options>] <file-or-url> [<run-args>]")
        W("Where:")
        W("    options ::= /nobuild | /norun | /silent | /fixup:member | /fixup:type")
        W("Description:")
        W("    Files must contain certain XML-like tags containing commands for building")
        W("    and running the program. For both C# and VB.NET files, default 'run' and")
        W("    'build' commands are available. Any command line argument after the file")
        W("    name is passed to the 'run' command. The '/nobuild' and '/norun' options")
        W("    allow skipping individual commands.")
        W("Examples:")
        W("    Clown BuildAndRun.vb")
        W("    Clown /norun BuildMe.cpp")
        W("    Clown /nobuild HardToRun.cs")
        W("    Clown Echo.cs Hello World")
        W("Command Examples (contained in file):")
        Dim cmd As CCommandInfo
        For Each cmd In info.Commands
            W("    " & cmd.Example)
        Next cmd
        W("Error Message:")
        W("    " & sMessage)
        Environment.Exit(1)
    End Sub

    Private Sub Log(ByVal obj As Object, ByVal fSilent As Boolean)
        If Not fSilent Then
            W(obj)
        End If
    End Sub

    Private Sub W(ByVal obj As Object)
        Console.WriteLine(obj)
    End Sub

    Private Function ReadLocalClownInfo(ByVal info As CClownInfo, ByVal params As CClownParams) As String
        Dim reader As New StreamReader(params.FileOrUrl)
        Return ReadClownInfo(reader, info)
    End Function

    Private Function ReadRemoteClownInfo(ByVal info As CClownInfo, params As CClownParams) As String
        Dim request As WebRequest = WebRequest.Create(params.FileOrUrl)
        Dim response As WebResponse = request.GetResponse()
        Dim stream As Stream = response.GetResponseStream()
        Dim reader As New StreamReader(stream)
        Return ReadClownInfo(reader, info)        
    End Function

    Private Function ReadClownInfo(ByVal reader As StreamReader, ByVal info As CClownInfo) As String
        Dim sb As New StringBuilder(4096)
        Try
            Dim sLine As String
            Dim sCmd As String = Nothing
            Do
                sLine = reader.ReadLine()
                If sLine Is Nothing Then
                    Exit Do
                End If
                Dim cmd As CCommandInfo
                For Each cmd In info.Commands
                    If cmd.CommandText Is Nothing Then
                        Dim sCmdText As String = ExtractBetween(sLine, cmd.GetOpener(), cmd.GetCloser())
                        If sCmdText Is Nothing AndAlso cmd.Name.Equals("build") Then
                            ' compatibility
                            sCmdText = ExtractBetween(sLine, "<clown>", "</clown>")
                        End If
                        cmd.CommandText = sCmdText
                    End If
                Next cmd
                sb.Append(sLine)
                sb.Append(Environment.NewLine)
            Loop
        Finally
            If Not (reader Is Nothing) Then
                reader.Close()
            End If
        End Try
        Return sb.ToString()
    End Function

    Private Sub SaveToTempFile(ByVal info As CClownInfo, params As CClownParams, ByVal sContents As String)
        Dim iLastSlash As Integer = params.FileOrUrl.LastIndexOf("/")
        If iLastSlash < 0 Then
            SyntaxExit(info, "Bad URL.")
        End If
        Dim sFileName As String = params.FileOrUrl.Substring(iLastSlash + 1)
        Dim sTempFile As String = Path.Combine(Environment.CurrentDirectory, sFileName)
        Dev.WriteTextFile(sTempFile, sContents)
        params.FileOrUrl = sTempFile
    End Sub

    Private Function GetExtension(ByVal params As CClownParams) As String
        Dim sExt As String = Nothing
        Dim iLastDot As Integer = params.FileOrUrl.LastIndexOf(".")
        If iLastDot >= 0 Then
            sExt = params.FileOrUrl.Substring(iLastDot)
        End If
        Return sExt
    End Function

    Private Function GuessDefaultCommands(ByVal info As CClownInfo, ByVal params As CClownParams) As Boolean
        Dim fRet As Boolean = False
        Dim sExt As String = GetExtension(params)
        If ".cs".Equals(sExt) OrElse ".vb".Equals(sExt) Then
            Dim sComp As String
            If sExt.Equals(".cs") Then
                sComp = "csc.exe"
            Else
                sComp = "vbc.exe"
            End If
            info.GetCommand("build").CommandText = sComp & " " & """" & params.FileOrUrl & """"
            info.GetCommand("run").CommandText = """" & params.FileOrUrl.Replace(sExt, ".exe") & """"
            Log("Using default commands for '" & sExt & "'.", params.Silent)
            fRet = True
        End If
        Return fRet
    End Function

    Private Sub FixupCode(ByVal info As CClownInfo, ByVal params As CClownParams)
        Log("Fixing up code.", params.Silent)
        Dim sCode As String = Dev.ReadTextFile(params.FileOrUrl)
        Dim sNewCode As String = FixupCode(info, params, sCode)
        ' oh, what a hack!
        Dim sExt As String = GetExtension(params)
        Dim sOldFile As String = params.FileOrUrl
        Dim sNewFile As String = sOldFile.Replace(sExt, ".fix" & sExt)
        Dev.WriteTextFile(sNewFile, sNewCode)
        Dim cmd As CCommandInfo
        cmd = info.GetCommand("build")
        cmd.CommandText = cmd.CommandText.Replace(sOldFile, sNewFile)
        cmd = info.GetCommand("run")
        cmd.CommandText = cmd.CommandText.Replace(".exe", ".fix.exe")
    End Sub

    Private Function FixupCode(ByVal info As CClownInfo, ByVal params As CClownParams, ByVal sCode As String) As String
        Dim sExt As String = GetExtension(params)
        Dim fixer As IFixupProvider = GetFixupProvider(sExt)
        params.FixupInfo.Imports = New String(){"System", "System.Collections", "System.Text"}
        Dim asCode() As String = fixer.FixupCode(params.FixupInfo, sCode)
        Return asCode(0)
    End Function

    Private Function GetFixupProvider(ByVal sExt As String) As IFixupProvider
        If ".cs".Equals(sExt) Then
            Return New CCSharpFixupProvider()
        ElseIf ".vb".Equals(sExt) Then
            Return New CVbNetFixupProvider()
        Else
            Throw New ArgumentException("No fixup provider available.")
        End If        
    End Function

    Private Sub RunCommands(ByVal info As CClownInfo, params As CClownParams)
        Dim cmd As CCommandInfo
        For Each cmd In info.Commands
            If params.SkipList.IndexOf(cmd.XmlTagName) >= 0 Then
                Log("Command '" & cmd.Name & "' skipped.", params.Silent)
            ElseIf cmd.CommandText Is Nothing Then
                Log("Command '" & cmd.Name & "' not set in file.", params.Silent)
            Else
                Dim sCmd As String = cmd.CommandText
                If cmd.UseArgs Then
                    sCmd &= " " & String.Join(" ", params.Arguments, params.FileArgIndex + 1, params.Arguments.Length - params.FileArgIndex - 1)
                End If
                Log("Command '" &cmd.Name & "': " & sCmd, params.Silent)
                Dim rc As Int32 = Shell(sCmd, , True)
                ' ... .NET compilers don't seem to return error codes
                If 0 <> rc Then
                    Log("Command '" & cmd.Name & "' failed with return code <" & rc & ">", params.Silent)
                    Exit For
                End If
            End If
        Next cmd
    End Sub

    Private Function ExtractBetween(ByVal s As String, ByVal sBefore As String, ByVal sAfter As String) As String
        Dim sRet As String = Nothing
        Dim i1 As Int32 = s.IndexOf(sBefore)
        If i1 >= 0 Then
            Dim i2 As Int32 = s.IndexOf(sAfter, i1 + 1)
            If i2 >= 0 Then
                Dim len As Int32 = sBefore.Length
                sRet = s.Substring(i1 + len, i2 - i1 - len).Trim()
            End If
        End If
        Return sRet
    End Function

End Module

Public Class CClownParams

    Private m_Arguments As String()
    Private m_FileOrUrl As String
    Private m_FileArgIndex As Integer
    Private m_IsUrl As Boolean
    Private m_Silent As Boolean
    Private m_SkipList As String
    Private m_FixupInfo As CFixupInfo

    Public Sub New(ByVal args() As String)
        m_Arguments = args
        m_FileOrUrl = Nothing
        m_FileArgIndex = -1
        m_IsUrl = False
        m_Silent = False
        m_SkipList = String.Empty
        m_FixupInfo = Nothing
    End Sub

    Public ReadOnly Property Arguments() As String()
        Get
            Return m_Arguments
        End Get
    End Property

    Public Property FileOrUrl() As String
        Get
            Return m_FileOrUrl
        End Get
        Set(ByVal newValue As String)
            m_FileOrUrl = newValue
        End Set
    End Property

    Public Property IsUrl() As Boolean
        Get
            Return m_IsUrl
        End Get
        Set(ByVal newValue As Boolean)
            m_IsUrl = newValue
        End Set
    End Property

    Public Property FileArgIndex() As Integer
        Get
            Return m_FileArgIndex
        End Get
        Set(ByVal newValue As Integer)
            m_FileArgIndex = newValue
        End Set
    End Property

    Public Property Silent() As Boolean
        Get
            Return m_Silent
        End Get
        Set(ByVal newValue As Boolean)
            m_Silent = newValue
        End Set
    End Property

    Public Property SkipList() As String
        Get
            Return m_SkipList
        End Get
        Set(ByVal newValue As String)
            m_SkipList = newValue
        End Set
    End Property

    Public Property FixupInfo() As CFixupInfo
        Get
            Return m_FixupInfo
        End Get
        Set(ByVal newValue As CFixupInfo)
            m_FixupInfo = newValue
        End Set
    End Property

End Class

Public Class CClownInfo

    Private m_Commands As ArrayList

    Public Sub New()
        m_Commands = New ArrayList()
    End Sub

    Public ReadOnly Property Commands() As ArrayList
        Get
            Return m_Commands
        End Get
    End Property

    Public Function GetCommand(ByVal sName As String) As CCommandInfo
        Dim info As CCommandInfo
        For Each info In m_Commands
            If 0 = String.Compare(info.Name, sName) Then
                Return info
            End If
        Next info
        Return Nothing
    End Function

    Public Function IsEmpty() As Boolean
        Dim cmd As CCommandInfo
        For Each cmd In m_Commands
            If Not (cmd.CommandText Is Nothing) Then
                Return False
            End If
        Next cmd
        Return True
    End Function

    Public Function IsComplete() As Boolean
        Dim cmd As CCommandInfo
        For Each cmd In m_Commands
            If cmd.CommandText Is Nothing Then
                Return False
            End If
        Next cmd
        Return True
    End Function

End Class

Public Class CCommandInfo

    Private m_Name As String
    Private m_XmlTagName As String
    Private m_Example As String
    Private m_CommandText As String
    Private m_UseArgs As Boolean

    Public Sub New(ByVal sName As String, ByVal sXmlTagName As String, ByVal sExample As String, Optional ByVal fUseArgs As Boolean = False)
        m_Name = sName
        m_XmlTagName = sXmlTagName
        m_Example = sExample
        m_UseArgs = fUseArgs
    End Sub

    Public ReadOnly Property Name() As String
        Get
            Return m_Name
        End Get
    End Property

    Public ReadOnly Property XmlTagName() As String
        Get
            Return m_XmlTagName
        End Get
    End Property

    Public ReadOnly Property Example() As String
        Get
            Return m_Example
        End Get
    End Property

    Public Property CommandText() As String
        Get
            Return m_CommandText
        End Get
        Set(ByVal newValue As String)
            m_CommandText = newValue
        End Set
    End Property

    Public ReadOnly Property UseArgs() As Boolean
        Get
            Return m_UseArgs
        End Get
    End Property

    Public Function GetOpener() As String
        Return "<" & m_XmlTagName & ">"
    End Function

    Public Function GetCloser() As String
        Return "</" & m_XmlTagName & ">"
    End Function

End Class

End Namespace
