// <clown-build> csc.exe /debug+ /r:Gregor.Core.dll,Gregor.AppCore.dll,Gregor.Inet.dll Downloader.cs </clown-build>
// <clown-run> Downloader.exe </clown-run>

using System;
using System.IO;
using Gregor.Core;
using Gregor.Inet;
using Gregor.Inet.Ftp;

[Module()]
public class M {

    private static bool s_MissingOnly;
    private static string s_RootUrl;
    private static string s_LocalRootFolder;
    private static string[] s_FilterPatterns;

    public static void Main(string[] args){
    
        if(args.Length < 3){
            SyntaxExit();
        }
        if(args[0] != "/all" && args[0] != "/missing"){
            SyntaxExit();
        }
        if(false == args[1].EndsWith("/")){
            SyntaxExit();
        }
        if(false == Directory.Exists(args[2])){
            SyntaxExit();
        }

        s_MissingOnly = (args[0] == "/missing");
        s_RootUrl = args[1];
        s_LocalRootFolder = args[2];
        s_FilterPatterns = new string[]{"*"};
        if(args.Length >= 4){
            s_FilterPatterns = args[3].Split(';');
        }

        try{
            CInternetConnection con = CInternetConnection.InternetConnection;
            con.KeepHandle = true;
            CFtpFolder folder = (CFtpFolder) con.ParseUrl(s_RootUrl);
            DownloadFolder(folder);
        }catch(Exception ex){
            W(ex);
        }
    }

    private static void SyntaxExit(){
        W("Syntax:");
        W("    Downloader /all ^ /missing <ftp-folder-url> <local-folder-path> [<filter-patterns>]");
        Environment.Exit(1);
    }

    private static void DownloadFolder(CFtpFolder folder){
        WorkItOut(folder.Update(null));
        W("Starting download from " + folder.FullUrl + " ...");

        string sFolderRelPath = GetFolderRelativePath(folder);
        string sLocalFolder = Path.Combine(s_LocalRootFolder, sFolderRelPath);
        sLocalFolder = sLocalFolder.Replace('/', '\\');
        W("    Downloading to " + sLocalFolder);
        if(false == Directory.Exists(sLocalFolder)){
            Directory.CreateDirectory(sLocalFolder);
        }

        foreach(CFtpFile file in folder.Files){
            if(ApplyFileFilter(file)){
                string sFile = Path.Combine(sLocalFolder, file.DisplayName);
                if(s_MissingOnly && File.Exists(sFile)){
                    W("        " + file.DisplayName + " skipped (exists).");
                }else{
                    W("        " + file.DisplayName);
                    WorkItOut(file.Download(null, sFile, FtpTransferMode.Unknown));
                }
            }
        }

        foreach(CFtpFolder subFolder in folder.Folders){
            DownloadFolder(subFolder);
        }
    }

    private static bool ApplyFileFilter(CFtpFile file){
        foreach(string sPattern in s_FilterPatterns){
            if(Flow.Like(file.DisplayName, sPattern)){
                return true;
            }
        }
        return false;
    }

    private static void WorkItOut(CInternetWorker worker){
        worker.Asynchronous = false;
        worker.Start();
    }

    private static string GetFolderRelativePath(CFtpFolder folder){
        return folder.FullUrl.Substring(s_RootUrl.Length);
    }

    private static void W(object obj){
        Console.WriteLine(obj);
    }

} // module M
