' <clown-build> vbc.exe /debug+ /r:Gregor.Core.dll,Gregor.AppCore.dll,Gregor.Inet.dll,System.Windows.Forms.dll,System.dll Publisher.vb </clown-build>
' <clown-run> Clown.exe </clown-run>
' 
' example:
' set SOURCE="C:\MyWebs\MySite"
' set TARGET="ftp://mysite.com/"
' Publisher /preview %SOURCE% %TARGET% /recurse *.htm*

Option Strict On

Imports System
Imports System.Collections
Imports System.Collections.Generic
Imports System.Collections.Specialized
Imports System.IO
Imports System.Text
Imports Gregor.Core
Imports Gregor.Core.Collections
Imports Gregor.Inet
Imports Gregor.Inet.CInternetConnection
Imports Gregor.Inet.Ftp

Namespace Publisher

Public Module M

    Private Const DATETIME_FORMAT As String = "yyyyMMdd.HHmmss"
    Private s_LogFile As String
    Private s_Preview As Boolean
    Private s_RootDirectory As String
    Private s_RootUrl As String
    Private s_Recurse As Boolean
    Private s_DirectoryTimestampLists As Dictionary(Of String, NCollection(Of DateTime))
    Private s_MessageBuffer As New ArrayList()

    Private Function GetFalse() As Boolean
        Return False
    End Function

    <STAThread()> _
    Public Sub Main(ByVal args() As String)
    
        Out.Trace.EnabledSensor = AddressOf GetFalse

        Try

            s_LogFile = Environment.GetEnvironmentVariable("PUBLISHER_LOGFILE")

            If args.Length < 4 _
            OrElse Not Flow.IsAmong(args(0), "/real", "/preview") _
            OrElse Not Flow.IsString(s_LogFile) _
            OrElse Not Flow.IsAmong(args(3), "/recurse", "/norecurse") Then
                Log("Publisher - .NET FTP Uploader")
                Log("Syntax:")
                Log("    Publisher /real|preview <absolute-local-source-dir> <remote-url> /recurse|norecurse [<file-patterns>]")
                Log("Requirements:")
                Log("    PUBLISHER_LOGFILE environment variable")
                Exit Sub
            End If
    
            Run(args)
    
        Catch ex As Exception
            Log(ex)
        End Try
    
    End Sub
    
    Private Sub Run(ByVal args() As String)
    
        s_Preview = args(0).Equals("/preview")
        s_RootDirectory = args(1)
        s_RootUrl = args(2)
        s_Recurse = args(3).Equals("/recurse")
        s_DirectoryTimestampLists = New Dictionary(Of String, NCollection(Of DateTime))()

        Dim asFilePatterns() As String = New String(){"*"}
        If args.Length > 4 Then
            Log("File Patterns: " & args(4))
            asFilePatterns = args(4).Split(";"c)
        End If

        Dim diRoot As DirectoryInfo = New DirectoryInfo(s_RootDirectory)
        If Not Char.IsLetter(s_RootDirectory.Chars(0)) _
        OrElse s_RootDirectory.Chars(1) <> ":"c Then
            s_RootDirectory = diRoot.FullName
            diRoot = New DirectoryInfo(s_RootDirectory)
            Log("Corrected local root directory to an absolute path: " & s_RootDirectory & ".")
        End If

        WalkDirectory(diRoot, asFilePatterns)

    End Sub
    
    Private Sub WalkDirectory(ByVal di As DirectoryInfo, ByVal asFilePatterns() As String)

        PublishDirectory(di, asFilePatterns)

        If s_Recurse Then
            Dim diSub As DirectoryInfo
            For Each diSub in di.GetDirectories()
                WalkDirectory(diSub, asFilePatterns)
            Next diSub
        End If

    End Sub
    
    Private Sub PublishDirectory(ByVal di As DirectoryInfo, ByVal asFilePatterns() As String)

        Dim files As IList(Of FileInfo) = GetFiles(di, asFilePatterns)

        If files.Count > 0 Then

            Dim sFullUrl As String = s_RootUrl & GetDirectoryRelativePath(di)
            Dim item As CFtpItemBase = InternetConnection.ParseUrl(sFullUrl)
            If Not TypeOf item Is CFtpFolder Then
                Log("The URL " & sFullUrl & " appearantly does not point to a folder.")
                Return
            End If
            Dim folder As CFtpFolder = DirectCast(item, CFtpFolder)

            If Not s_Preview Then
                Log("Checking existance of " & folder.FullUrl & " ...")
                If Not EnsureDirectoryExists(folder) Then
                    Return ' ...
                End If
            End If

            Dim sBytes As String = FormatBytes(GetFileSizeTotal(files))
            Log("Processing " & di.FullName & " ...")
            Log("    Uploading to " & sFullUrl & " (" & files.Count & " files, " & sBytes & " bytes) ...")

            For Each sMsg As String in s_MessageBuffer
                Log(sMsg)
            Next sMsg
            s_MessageBuffer.Clear()

            Dim file As FileInfo
            Dim cFailed As Int32 = 0

            For Each file In files
                If PublishFile(folder, file) Then
                    If Not s_Preview Then
                        SetFilePublicationTime(file, DateTime.Now)
                    End If
                Else
                    cFailed += 1
                    Log("        - publishing " & file.Name & " FAILED!")
                    If Not s_Preview Then
                        SetFilePublicationTime(file, DateTime.MinValue)
                    End If
                End If
            Next file

            If cFailed > 0 Then
                Log("        " & cFailed & " of " & files.Count & " file(s) FAILED.")
            End If

        End If
    
    End Sub

    Private Function EnsureDirectoryExists(ByVal folder As CFtpFolder) As Boolean
        Dim fRet As Boolean = False
        Dim worker As CInternetWorker = folder.Update(Nothing)
        fRet = WorkItOut(worker)
        If fRet Then
            Log("Remote folder " & folder.FullUrl & " exists.")
        Else
            Log("Remote folder " & folder.FullUrl & " does not exist.")
            Dim parentFolder As CFtpFolder = folder.ParentFolder
            If Not (parentFolder Is Nothing) _
            AndAlso EnsureDirectoryExists(parentFolder) Then
                Log("Trying to create " & folder.DisplayName & " on " & parentFolder.FullUrl & " ...")
                worker = parentFolder.CreateDirectory(Nothing, folder.DisplayName)
                fRet = WorkItOut(worker)
                If fRet Then                    
                    Log("Successfully created " & folder.FullUrl & ".")
                    Log("Refreshing " & folder.FullUrl & " ...")
                    worker = folder.Update(Nothing)
                    fRet = WorkItOut(worker)
                    If Not fRet Then
                        Log("Refreshing " & folder.FullUrl & " failed.")
                    End If
                Else
                    Log("Unable to create " & folder.FullUrl & ".")
                End If
            End If
        End If
        Return fRet
    End Function

    Private Function PublishFile(ByVal folder As CFtpFolder, ByVal fi As FileInfo) As Boolean

        Dim fRet As Boolean = False

        Log("        Publishing " & fi.Name & " (" & FormatBytes(fi.Length) & " bytes) ...")
    
        If s_Preview Then
            fRet = True
        Else
            Dim worker As CInternetWorker = folder.UploadFile(Nothing, fi.FullName, fi.Name, FtpTransferMode.Unknown)
            fRet = WorkItOut(worker)
        End If

        Return fRet

    End Function

    Private Function WorkItOut(ByVal worker As CInternetWorker) As Boolean
        worker.Asynchronous = False
        worker.Start()
        Return worker.Succeeded
    End Function
    
    Private Function GetFiles(ByVal di As DirectoryInfo, ByVal asFilePatterns() As String) As IList(Of FileInfo)
    
        Dim list As New List(Of FileInfo)()

        For Each sFilePattern As String In asFilePatterns
            For Each fi As FileInfo In di.GetFiles(sFilePattern)
                If 0 <> String.Compare(fi.Name, s_LogFile, True) Then
                    Dim stamp As DateTime = GetFilePublicationTime(fi)
                    If fi.LastWriteTime >= stamp OrElse fi.CreationTime >= stamp Then
                        list.Add(fi)
                    End If
                End If
            Next fi
        Next sFilePattern

        Return list
    
    End Function

    Private Function GetFileSizeTotal(ByVal files As IList(Of FileInfo)) As Long
        Dim ret As Long = 0
        For Each fi As FileInfo In files
            ret += fi.Length
        Next fi
        Return ret
    End Function

    Private Function FormatBytes(ByVal n As Long) As String
        Return string.Format("{0:n0}", n)
    End Function

    Private Function GetFilePublicationTime(ByVal fi As FileInfo) As DateTime
    
        Dim stamps As NCollection(Of DateTime) = LoadDirectoryTimestampList(fi.Directory)

        If stamps.Exists(fi.Name) Then
            Return stamps.Item(fi.Name)
        Else
            s_MessageBuffer.Add("        - no timestamp available for file '"& fi.Name &"'.")
            Return DateTime.MinValue
        End If

    End Function

    Private Sub SetFilePublicationTime(ByVal fi As FileInfo, ByVal dt As DateTime)
        
        Dim stamps As NCollection(Of DateTime) = LoadDirectoryTimestampList(fi.Directory)

        If stamps.Exists(fi.Name) Then
            stamps.Item(fi.Name) = dt
        Else
            stamps.Add(dt, fi.Name)
        End If

        SaveDirectoryTimestampList(fi.Directory, stamps)

    End Sub
    
    Private Function LoadDirectoryTimestampList(ByVal di As DirectoryInfo) As NCollection(Of DateTime)

        If s_DirectoryTimestampLists.ContainsKey(di.FullName) Then

            Return s_DirectoryTimestampLists.Item(di.FullName)

        End If

        Dim ret As New NCollection(Of DateTime)()

        Dim fiStamp As FileInfo = GetDirectoryTimestampFile(di)

        If fiStamp.Exists Then
        
            Dim lines As StringCollection = FileUtil.ReadTextFileLines(fiStamp.FullName)

            For Each sLine As String In lines

                Dim iColon As Int32 = sLine.IndexOf(":"c)
                If iColon > 0 Then ' yes, > 0
                    Dim sKey As String = sLine.Substring(0, iColon)
                    Dim sValue As String = sLine.Substring(iColon + 1)
                    Try
                        Dim dt As DateTime = DateTime.ParseExact(sValue, DATETIME_FORMAT, Nothing)
                        If ret.Exists(sKey) Then
                            Log("        - duplicate timestamp for file '" & sKey & "', using the smaller one.")
                            If DateTime.Compare(dt, ret.Item(sKey)) < 0 Then
                                ret.Item(sKey) = dt
                            End If
                        Else
                            ret.Add(dt, sKey)
                        End If
                    Catch
                        Log("        - parsing timestamp for file '" & sKey & "' failed.")
                    End Try
                End If

            Next sLine

        Else

            Log("        - timestamp file does not exist in directory '" & di.Name & "'.")

        End If

        s_DirectoryTimestampLists.Add(di.FullName, ret)

        Return ret
        
    End Function
    
    Private Sub SaveDirectoryTimestampList(ByVal di As DirectoryInfo, ByVal stamps As NCollection(Of DateTime))

        Dim fiStamp As FileInfo = GetDirectoryTimestampFile(di)

        Dim sb As New StringBuilder(stamps.Count * 16)

        Dim sNow As String = DateTime.Now.ToString(DATETIME_FORMAT)
        sb.Append(sNow)
        sb.Append(Environment.NewLine)

        For Each sKey As String In Walk.SortDirect(stamps.GetKeys())
            Dim dt As DateTime = stamps.Item(sKey)
            Dim sValue As String = dt.ToString(DATETIME_FORMAT)
            sb.Append(sKey)
            sb.Append(":")
            sb.Append(sValue)
            sb.Append(Environment.NewLine)
        Next sKey

        FileUtil.WriteTextFile(fiStamp.FullName, sb.ToString())

    End Sub

    Private Function GetDirectoryTimestampFile(ByVal di As DirectoryInfo) As FileInfo

        Dim sPath As String = Path.Combine(di.FullName, s_LogFile)
        Return New FileInfo(sPath)

    End Function
    
    Private Function GetDirectoryRelativePath(ByVal di As DirectoryInfo) As String
    
        Dim s As String = di.FullName.Substring(s_RootDirectory.Length)
    
        s = s.Replace("\", "/")
        If s.StartsWith("/") Then s = s.Substring(1)
        If Not s.EndsWith("/") Then s = s & "/"
        If s.Equals("/") Then s = String.Empty
    
        Return s
    
    End Function
    
    Private Sub Log(ByVal o As Object)
    
        Console.WriteLine(o)
    
    End Sub

End Module

End Namespace
