// <clown-build> csc.exe /r:Gregor.Core.dll ShowDirectorySize.cs </clown-build>
// <clown-run> ShowDirectorySize </clown-run>

using System;
using System.IO;
using Gregor.Core;

namespace Gregor {

[Module()]
public class M {

    public static void Main(string[] args){
        if(args.Length < 2){
            Console.WriteLine("Syntax:");
            Console.WriteLine("    ShowDirectorySize <root-dir> <max-log-level> [<log-threshold>]");
            Environment.Exit(1);
        }
        try{
            DirectoryInfo diRoot = new DirectoryInfo(args[0]);
            int maxLogLevel = int.Parse(args[1]);
            long logThreshold = 0;
            if(args.Length > 2){
                logThreshold = long.Parse(args[2]);
            }
            ProcessDirectory(diRoot, 0, maxLogLevel, logThreshold);
        }catch(Exception ex){
            Console.WriteLine(ex);
        }
    }

    private static long ProcessDirectory(DirectoryInfo di, int level, int maxLogLevel, long logThreshold){
        long sum = 0;
        foreach(FileInfo fi in di.GetFiles()){
            sum += fi.Length;
        }
        DirectoryInfo[] childDirs = di.GetDirectories();
        foreach(DirectoryInfo diChild in Walk.Sort(childDirs, "Name")){
            sum += ProcessDirectory(diChild, level + 1, 0, logThreshold);
        }
        if(level <= maxLogLevel && sum >= logThreshold){
            Console.WriteLine("{0:s}{1:s}: {2:n0}", new string(' ', level * 2), di.Name, sum);
        }
        if(level + 1 <= maxLogLevel){
            foreach(DirectoryInfo diChild in Walk.Sort(childDirs, "Name")){
                ProcessDirectory(diChild, level + 1, maxLogLevel, logThreshold);
            }
        }
        return sum;
    }

} // class M

} // namespace Gregor
