// <action:OutputWindow>
// <shell:csc.exe /t:winexe $Vars.DocName%>
// <start:$Vars.DocExePath%>

using System;
using System.Collections;
using System.Text;
using System.Diagnostics;
using System.Drawing; 
using System.Windows.Forms;
using System.Runtime.InteropServices;

public class C {

    internal delegate bool EnumWindowsCallback(IntPtr hWnd, IntPtr lParam);

    [DllImport("user32")]
    extern static internal int EnumWindows(EnumWindowsCallback callback, IntPtr lParam);

    [DllImport("user32")]
    extern static internal bool IsWindowVisible(IntPtr hWnd);

    [DllImport("user32")]
    extern static internal int GetWindowText(IntPtr hWnd, StringBuilder sb, int maxCount);

    [DllImport("user32")]
    extern static internal int ShowWindow(IntPtr hWnd, int flags);

    internal const int SW_HIDE = 0;
    internal const int SW_SHOW = 5;

    public static void Main(string[] args){

        F frm = new F();
        Application.Run(frm);
    }

} // class C

internal class CWindow : IComparable {

    private IntPtr m_Handle;
    private string m_Text;

    public CWindow(IntPtr hWnd, string sText){
        m_Handle = hWnd;
        m_Text = sText;
    }

    public IntPtr Handle{
        get{
            return m_Handle;
        }
    }

    public string Text{
        get{
            return m_Text;
        }
    }

    public override string ToString(){
        return this.Text;
    }

    public void Hide(){
        C.ShowWindow(m_Handle, C.SW_HIDE);
    }

    public void Show(){
        C.ShowWindow(m_Handle, C.SW_SHOW);
    }

    /* private implement */ int IComparable.CompareTo(object obj){
        CWindow that = (CWindow) obj;
        //return this.m_Handle.ToInt32() - that.m_Handle.ToInt32();
        return this.m_Text.CompareTo(that.m_Text);
    }

} // class CWindow

internal class F : Form {

    private ListBox m_ListBox;
    private Button m_UpButton;
    private Button m_DownButton;
    private Button m_ApplyButton;
    private StringBuilder m_Buffer;

    public F(){
        this.Text = "Rearrange Taskbar";
        this.ShowInTaskbar = false;
        this.TopLevel = this.TopMost = true;
        m_ListBox = new ListBox();
        m_ListBox.Bounds = new Rectangle(5, 5, this.ClientRectangle.Width - 95, this.ClientRectangle.Height - 10);
        m_ListBox.Anchor = AnchorStyles.Left | AnchorStyles.Right | AnchorStyles.Top | AnchorStyles.Bottom;
        m_ListBox.SelectedIndexChanged += new EventHandler(this.HandleListBoxSelectedIndexChanged);
        this.Controls.Add(m_ListBox);
        m_UpButton = new Button();
        m_UpButton.Text = "Up";
        m_UpButton.Bounds = new Rectangle(this.ClientRectangle.Width - 85, 5, 80, 22);
        m_UpButton.Anchor = AnchorStyles.Right | AnchorStyles.Top;
        m_UpButton.Enabled = false;
        m_UpButton.Click += new EventHandler(this.HandleUpButtonClick);
        this.Controls.Add(m_UpButton);
        m_DownButton = new Button();
        m_DownButton.Text = "Down";
        m_DownButton.Bounds = new Rectangle(this.ClientRectangle.Width - 85, 30, 80, 22);
        m_DownButton.Anchor = AnchorStyles.Right | AnchorStyles.Top;
        m_DownButton.Enabled = false;
        m_DownButton.Click += new EventHandler(this.HandleDownButtonClick);
        this.Controls.Add(m_DownButton);
        m_ApplyButton = new Button();
        m_ApplyButton.Text = "Apply";
        m_ApplyButton.Bounds = new Rectangle(this.ClientRectangle.Width - 85, 55, 80, 22);
        m_ApplyButton.Anchor = AnchorStyles.Right | AnchorStyles.Top;
        m_ApplyButton.Enabled = true;
        m_ApplyButton.Click += new EventHandler(this.HandleApplyButtonClick);
        this.Controls.Add(m_ApplyButton);
        m_Buffer = new StringBuilder(1024);
    }

    private void HandleListBoxSelectedIndexChanged(object sender, EventArgs e){
        try{
            int idx = m_ListBox.SelectedIndex;
            m_UpButton.Enabled = (idx > 0);
            m_DownButton.Enabled = (idx < m_ListBox.Items.Count - 1); 
        }catch(Exception ex){
            MessageBox.Show(ex.ToString());
        }
    }

    private void HandleUpButtonClick(object sender, EventArgs e){
        try{
            int idx = m_ListBox.SelectedIndex;
            if(idx > 0){
                object obj = m_ListBox.Items[idx];
                m_ListBox.Items.RemoveAt(idx);
                m_ListBox.Items.Insert(idx - 1, obj);
                m_ListBox.SelectedItem = obj;
            }
        }catch(Exception ex){
            MessageBox.Show(ex.ToString());
        }
    }

    private void HandleDownButtonClick(object sender, EventArgs e){
        try{
            int idx = m_ListBox.SelectedIndex;
            if(idx < m_ListBox.Items.Count - 1){
                object obj = m_ListBox.Items[idx];
                m_ListBox.Items.RemoveAt(idx);
                m_ListBox.Items.Insert(idx + 1, obj);
                m_ListBox.SelectedItem = obj;
            }
        }catch(Exception ex){
            MessageBox.Show(ex.ToString());
        }
    }

    private void HandleApplyButtonClick(object sender, EventArgs e){
        try{
            foreach(CWindow window in m_ListBox.Items){
                window.Hide();
            }
            foreach(CWindow window in m_ListBox.Items){
                window.Show();
            }
        }catch(Exception ex){
            MessageBox.Show(ex.ToString());
        }
    }

    protected override void OnLoad(EventArgs e){
        try{
            C.EnumWindows(new C.EnumWindowsCallback(HandleEnumWindowsCallback), IntPtr.Zero);
            ArrayList.Adapter(m_ListBox.Items).Sort();
        }catch(Exception ex){
            MessageBox.Show(ex.ToString());
        }finally{
            base.OnLoad(e);
        }
    }

    private bool HandleEnumWindowsCallback(IntPtr hWnd, IntPtr lParam){
        if(C.IsWindowVisible(hWnd)){
            m_Buffer.Remove(0, m_Buffer.Length);
            if(0 < C.GetWindowText(hWnd, m_Buffer, m_Buffer.Capacity)){
                string s = m_Buffer.ToString();
                m_ListBox.Items.Add(new CWindow(hWnd, s));
            }
        }
        return true;
    }

} // class F