' <clown-build> vbc.exe /r:ICSharpCode.SharpZipLib.dll Zipper.vb </clown-build>
' <clown-run> Zipper.exe </clown-run>

Option Strict On

Imports System
Imports System.Collections
Imports System.IO
Imports System.Text
Imports ICSharpCode.SharpZipLib.Checksums
Imports ICSharpCode.SharpZipLib.Zip

Namespace Zipper

Friend Enum FilePathMode
    NameOnly
    Relative
    Absolute
End Enum

Public Module MStart

Private m_BaseDirectoryPath As String ' for relative entry names
Private m_FilePathMode As FilePathMode

Public Sub Main(ByVal args() As String)
    ' hello
    Log("Zipper - Driving #ZipLib")
    ' check
    If args.Length < 3 OrElse args(0) <> "/name" AndAlso args(0) <> "/rel" AndAlso args(0) <> "/abs" Then
        Log("Syntax:")
        Log("    Zipper /name | /rel | /abs <source-dir> <zip-file> [<file-patterns>]")
        Exit Sub
    End If
    ' set file path mode
    If args(0) = "/name" Then
        m_FilePathMode = FilePathMode.NameOnly
        Log("Using file names only")
    ElseIf args(0) = "/rel" Then
        m_FilePathMode = FilePathMode.Relative
        Log("Using relative file paths")
    ElseIf args(0) = "/abs" Then
        m_FilePathMode = FilePathMode.Absolute
        Log("Using absolute file paths")
    End If
    ' filter files?
    Dim asFilePatterns() As String = New String(){"*"}
    If args.Length > 3 Then
        Log("File Patterns: " & args(3))
        asFilePatterns = args(3).Split(";"c)
    End If
    ' so
    Dim outStream As ZipOutputStream
    Try
        Dim dir As New DirectoryInfo(args(1))
        If dir.Parent Is Nothing Then
            m_BaseDirectoryPath = dir.FullName
        Else
            m_BaseDirectoryPath = dir.Parent.FullName
        End If
        If Not m_BaseDirectoryPath.EndsWith("\") Then
            m_BaseDirectoryPath &= "\"
        End If
        Dim fileStream As Stream = File.Create(args(2) & ".tmp")
        outStream = New ZipOutputStream(fileStream)
        outStream.SetLevel(6)
        Dim checkSum As New Crc32()
        ProcessDirectory(dir, outStream, checkSum, asFilePatterns)
    Catch ex As Exception
        Log(ex)
    Finally
        If Not (outStream Is Nothing) Then
            ' finish the output stream
            outStream.Finish()
            outStream.Close()
            ' maybe copy over real file
            If Not CompareFileContent(args(2), args(2) & ".tmp") Then
                File.Copy(args(2) & ".tmp", args(2), True)
            End If
            ' delete temporary file
            File.Delete(args(2) & ".tmp")
            ' set timestamp
            File.SetLastWriteTime(args(2), DateTime.Now)
        End If
    End Try
    ' done
    Log("Done")
End Sub

Private Function CompareFileContent(ByVal sFile1 As String, ByVal sFile2 As String) As Boolean
    Dim fRet As Boolean = False
    If File.Exists(sFile1) AndAlso File.Exists(sFile2) Then
        ' get contents
        Dim ab1(), ab2() As Byte
        Try
            ab1 = ReadFile(sFile1)
            ab2 = ReadFile(sFile2)
        Catch ex As Exception
            Log(ex)
            Exit Function
        End Try
        ' compare
        If ab1.Length = ab2.Length Then
            fRet = True
            Dim i As Integer
            For i = 0 To ab1.Length - 1
                If ab1(i) <> ab2(i) Then
                    fRet = False
                    Exit For
                End If
            Next i
        End If
    End If
    Return fRet
End Function

Private Function ReadFile(ByVal sFile As String) As Byte()
    Dim ab() As Byte
    Dim stream As FileStream = Nothing
    Try
        stream = File.OpenRead(sFile)
        ab = New Byte(Convert.ToInt32(stream.Length)){}
        stream.Read(ab, 0, ab.Length)
    Finally
        If Not (stream Is Nothing) Then
            stream.Close()
        End If
    End Try
    Return ab
End Function

Private Sub ProcessDirectory(ByVal dir As DirectoryInfo, ByVal outStream As ZipOutputStream, ByVal checkSum As ICheckSum, ByVal asFilePatterns() As String)
    ' zip files in directory
    ZipDirectoryContents(dir, outStream, checkSum, asFilePatterns)
    ' recurse
    Dim subDir As DirectoryInfo
    For Each subDir In dir.GetDirectories()
        ProcessDirectory(subDir, outStream, checkSum, asFilePatterns)
    Next subDir
End Sub

Private Sub ZipDirectoryContents(ByVal dir As DirectoryInfo, ByVal outStream As ZipOutputStream, ByVal checkSum As ICheckSum, ByVal asFilePatterns() As String)
    ' say
    Log("Zipping " & dir.FullName & " ...")
    ' get files
    Dim files() As FileInfo = GetFiles(dir, asFilePatterns)
    ' zip them
    If files.Length = 0 AndAlso m_FilePathMode <> FilePathMode.NameOnly Then
        ' empty leaf directory?
        If dir.GetDirectories().Length = 0 Then
            ZipEmptyDirectory(dir, outStream, checkSum)
        End If
    Else
        ' walk
        Dim fil As FileInfo
        For Each fil In files
            ZipFile(fil, outStream, checkSum)
        Next fil
    End If
End Sub

Private Sub ZipEmptyDirectory(ByVal dir As DirectoryInfo, ByVal outStream As ZipOutputStream, ByVal checkSum As ICheckSum)
    Dim sName As String = GetZipEntryFileName(dir)
    sName &= "\DeleteMe.txt"
    Dim sComment As String = "This is a dummy file that can be deleted." & Environment.NewLine _
                           & "It exists so that the directory '" & dir.Name & "'"  & Environment.NewLine _
                           & "is retained in the zip archive." & Environment.NewLine
    Dim ab() As Byte = Encoding.Default.GetBytes(sComment) ' {68, 69, 76, 69, 84, 69, 32, 77, 69}
    AddZipEntry(sName, dir.LastWriteTime, ab, outStream, checkSum)
End Sub

Private Sub ZipFile(ByVal fil As FileInfo, ByVal outStream As ZipOutputStream, ByVal checkSum As ICheckSum)
    Dim inStream As FileStream = Nothing
    Try
        ' read file
        inStream = fil.OpenRead()
        Dim ab() As Byte = New Byte(Convert.ToInt32(inStream.Length - 1)){}
        inStream.Read(ab, 0, ab.Length)
        ' add zip entry for file
        Dim sName As String = GetZipEntryFileName(fil)
        AddZipEntry(sName, fil.LastWriteTime, ab, outStream, checkSum)
    Finally
        If Not (inStream Is Nothing) Then
            inStream.Close()
        End If
    End Try
End Sub

Private Function AddZipEntry(ByVal sName As String, ByVal timeStamp As DateTime, ByVal ab() As Byte, ByVal outStream As ZipOutputStream, ByVal checkSum As ICheckSum) As ZipEntry
    ' create new zip entry
    Dim entry As New ZipEntry(sName)
    entry.DateTime = timeStamp
    entry.Size = ab.Length
    ' set checksum
    checkSum.Reset()
    checkSum.Update(ab)
    entry.Crc = checkSum.Value
    ' add entry to output stream
    outStream.PutNextEntry(entry)
    outStream.Write(ab, 0, ab.Length)    
    ' done
    Return entry
End Function

Private Function GetFiles(ByVal di As DirectoryInfo, ByVal asFilePatterns() As String) As FileInfo()
    Dim sFilePattern As String, fi As FileInfo, list As New ArrayList()
    For Each sFilePattern In asFilePatterns
        For Each fi In di.GetFiles(sFilePattern)
            list.Add(fi)
        Next fi
    Next sFilePattern
    Dim files() As FileInfo = New FileInfo(list.Count - 1){}
    list.CopyTo(files, 0)
    Return files
End Function

Private Function GetZipEntryFileName(ByVal fsi As FileSystemInfo) As String
    Select Case m_FilePathMode
        Case FilePathMode.NameOnly
            Return fsi.Name
        Case FilePathMode.Relative
            Return fsi.FullName.Substring(m_BaseDirectoryPath.Length)
        Case FilePathMode.Absolute
            Return fsi.FullName
    End Select
End Function

Private Sub Log(ByVal o As Object)
    Console.WriteLine(o)
End Sub

End Module

End Namespace
